@@ -25,6 +25,25 @@
 Drug information, with an emphasis on psychotropic drugs, including translating
 specific to generic names.
 
+**Examples**
+
+Test within Python:
+
+.. code-block:: python
+
+    from cardinal_pythonlib.drugnames import drug_name_to_generic
+
+    drug_name_to_generic("UNKNOWN")
+    drug_name_to_generic("UNKNOWN", unknown_to_default=True)
+    drug_names_to_generic([
+        "citalopram", "Citalopram", "Cipramil", "Celexa",
+        "olanzepine",  # typo
+        "dextroamphetamine",
+        "amitryptyline",
+    ])
+
+**Antidepressants**
+
 As of 2018-07-01, this is a functional superset of the SLAM
 antidepressant-finding SQL (see ``dep_med_v1``), though mainly a superset in
 non-antidepressant respects; the only antidepressants it adds are:
@@ -79,20 +98,7 @@ Compare that against the output of:
     [x.generic_name for x in all_drugs_where(slam_antidepressant_finder=True,
                                              include_categories=True)]
 
-Test within Python:
-
-.. code-block:: python
-
-    from cardinal_pythonlib.drugnames import drug_name_to_generic
-
-    drug_name_to_generic("UNKNOWN")
-    drug_name_to_generic("UNKNOWN", unknown_to_default=True)
-    drug_names_to_generic([
-        "citalopram", "Citalopram", "Cipramil", "Celexa",
-        "olanzepine",  # typo
-        "dextroamphetamine",
-        "amitryptyline",
-    ])
+**Using this code from R via reticulate**
 
 Test within R:
 
@@ -101,39 +107,52 @@ Test within R:
     # -------------------------------------------------------------------------
     # Load libraries
     # -------------------------------------------------------------------------
+    
+    RUN_ONCE_ONLY <- '
+        library(devtools)
+        devtools::install_github("rstudio/reticulate")  # get latest version
+    '
 
     library(data.table)
     library(reticulate)
 
     # -------------------------------------------------------------------------
-    # Struggle with reticulate bugs to import a module
+    # Set up reticulate
     # -------------------------------------------------------------------------
 
-    VENV <- "~/dev/venvs/cardinal_pythonlib"
-    PYTHON_VERSION <- "python3.5"
-    CARDINAL_PYTHONLIB_BASEDIR <- ifelse(
+    VENV <- "~/dev/venvs/cardinal_pythonlib"  # or your preferred virtualenv
+    PYTHON_EXECUTABLE <- ifelse(
         .Platform$OS.type == "windows",
-        file.path(VENV, "lib", "site-packages/cardinal_pythonlib"),
-        file.path(VENV, "lib", PYTHON_VERSION, "site-packages/cardinal_pythonlib")
+        file.path(VENV, "Scripts", "python.exe"),  # Windows
+        file.path(VENV, "bin", "python")  # Linux
     )
-
-    reticulate::use_virtualenv(VENV, required=TRUE)
+    reticulate::use_python(PYTHON_EXECUTABLE, required=TRUE)
     # ... it is CRITICAL to use required=TRUE, or it might fail silently
+    
+    # Unnecessary now reticulate::use_python() works:
+    # 
+    # PYTHON_VERSION <- "python3.5"
+    # CARDINAL_PYTHONLIB_BASEDIR <- ifelse(
+    #     .Platform$OS.type == "windows",
+    #     file.path(VENV, "lib", "site-packages/cardinal_pythonlib"),
+    #     file.path(VENV, "lib", PYTHON_VERSION, "site-packages/cardinal_pythonlib")
+    # )
+    # reticulate::use_virtualenv(VENV, required=TRUE)
+    #
+    # cpl_fileops <- reticulate::import_from_path("fileops", CARDINAL_PYTHONLIB_BASEDIR)
+    # cpl_drugs <- reticulate::import_from_path("drugs", file.path(CARDINAL_PYTHONLIB_BASEDIR, "psychiatry"))
+    #
+    # ... this is NOT WORKING properly; dotted imports via reticulate::import() fail; also, imports from
+    # within the Python code fail even if you use reticulate::import_from_path(); this suggests the virtualenv is not set up
+    # properly; use reticulate::use_python() instead.
 
-    # Note that reticulate doesn't cope with dotted package names:
-    cardinal_pythonlib <- reticulate::import("cardinal_pythonlib")  # works
-    cpl_drugnames <- reticulate::import("cardinal_pythonlib.drugnames")  # FAILS
-    cpl_fileops <- reticulate::import("cardinal_pythonlib.fileops")  # FAILS
-    # ... despite "import cardinal_pythonlib.fileops" working fine in Python
+    # -------------------------------------------------------------------------
+    # Import Python modules
+    # -------------------------------------------------------------------------
 
-    # So, nasty hack:
-    cpl_fileops <- reticulate::import_from_path("fileops", CARDINAL_PYTHONLIB_BASEDIR)
-
-    # And the module we care about now:
-    cpl_drugs <- reticulate::import_from_path("drugs", file.path(CARDINAL_PYTHONLIB_BASEDIR, "psychiatry"))
-
-    # Or, for development:
-    cpl_drugs <- reticulate::import_from_path("drugs", "~/Documents/code/cardinal_pythonlib/cardinal_pythonlib/psychiatry")
+    cardinal_pythonlib <- reticulate::import("cardinal_pythonlib")
+    cpl_fileops <- reticulate::import("cardinal_pythonlib.fileops")
+    cpl_drugs <- reticulate::import("cardinal_pythonlib.psychiatry.drugs")
 
     # -------------------------------------------------------------------------
     # Do something useful
@@ -150,14 +169,23 @@ Test within R:
     )
     dt[, drug_generic := cpl_drugs$drug_names_to_generic(drug)]
     dt[, is_antidepressant := cpl_drugs$drug_names_match_criteria(
-            drug_generic, names_are_generic=TRUE,
+            drug_generic, 
+            names_are_generic=TRUE,
             antidepressant=TRUE)]
     dt[, is_antidepressant_not_ssri := cpl_drugs$drug_names_match_criteria(
-            drug_generic, names_are_generic=TRUE,
-            antidepressant=TRUE, ssri=FALSE)]
+            drug_generic, 
+            names_are_generic=TRUE,
+            antidepressant=TRUE,
+            ssri=FALSE)]
     dt[, is_conventional_antidepressant := cpl_drugs$drug_names_match_criteria(
-            drug_generic, names_are_generic=TRUE,
+            drug_generic, 
+            names_are_generic=TRUE,
             conventional_antidepressant=TRUE)]
+    dt[, slam_antidepressant_finder := cpl_drugs$drug_names_match_criteria(
+            drug_generic, 
+            names_are_generic=TRUE,
+            slam_antidepressant_finder=TRUE,
+            include_categories=TRUE)]
 
 """  # noqa
 
@@ -165,6 +193,13 @@ import re
 from typing import Dict, List, Optional, Union
 
 
+# =============================================================================
+# Regex constants
+# =============================================================================
+
+WILDCARD = ".*"  # if re.DOTALL is set, this also matches newlines
+WB = WORD_BOUNDARY = r"\b"
+
 # =============================================================================
 # Class to capture drug information
 # =============================================================================
@@ -177,6 +212,7 @@ class Drug(object):
             alternatives: List[str] = None,
             category_not_drug: bool = False,
             add_preceding_wildcards: bool = True,
+            add_preceding_word_boundary: bool = True,
             add_following_wildcards: bool = True,
             # Psychiatry
             psychotropic: bool = None,  # special; can be used as override if False  # noqa
@@ -227,6 +263,11 @@ class Drug(object):
         Initialize and determine/store category knowledge.
 
         "alternatives" can include regexes (as text).
+
+        We add front/back wildcards by default; this handles all situations
+        like "depot X", etc. We also add a preceding word boundary (after the
+        wildcard); thus the usual transformation is ``XXX`` -> ``.*\bXXX.*``.
+
         """
         # ---------------------------------------------------------------------
         # Name handling
@@ -244,18 +285,16 @@ class Drug(object):
             raise ValueError("Bad generic_name: {!r}".format(generic))
         self.alternatives = alternatives or []  # type: List[str]
         possibilities = []  # type: List[str]
-        wildcard = ".*"  # see re.DOTALL below, which makes this match newlines
         for p in list(set(self.all_generics + self.alternatives)):
-            # We add front/back wildcards by default; this handles all
-            # situations like "depot X", etc.
-            if add_preceding_wildcards and not p.startswith(wildcard):
-                p = wildcard + p
-            if add_following_wildcards and not p.endswith(wildcard):
-                p = p + wildcard
+            if add_preceding_word_boundary and not p.startswith(WB):
+                p = WB + p
+            if add_preceding_wildcards and not p.startswith(WILDCARD):
+                p = WILDCARD + p
+            if add_following_wildcards and not p.endswith(WILDCARD):
+                p = p + WILDCARD
             possibilities.append(p)
         regex_text = "|".join("(?:" + x + ")" for x in possibilities)
-        self.regex = re.compile(regex_text,
-                                re.IGNORECASE | re.VERBOSE | re.DOTALL)
+        self.regex = re.compile(regex_text, re.IGNORECASE | re.DOTALL)
 
         # ---------------------------------------------------------------------
         # Things we know about psychotropics
@@ -618,7 +657,7 @@ DRUGS = [
     # -------------------------------------------------------------------------
     Drug(
         "tricyclic_antidepressant",
-        [".*tricyclic.*", ".*tca.*"],
+        ["tricyclic.*", "tca" + WB],
         tricyclic_antidepressant=True,
         slam_antidepressant_finder=True,
         category_not_drug=True,
